@@ -20,7 +20,7 @@
 #' failed, and you need to take a look at the log files under that directory.
 #'
 #' The time to finish the check is recorded for each package. As the check goes
-#' on, the total remaing time will be roughly estimated via \code{n *
+#' on, the total remaining time will be roughly estimated via \code{n *
 #' mean(times)}, where \code{n} is the number of packages remaining to be
 #' checked, and \code{times} is a vector of elapsed time of packages that have
 #' been checked.
@@ -75,6 +75,7 @@
 #' @param src_dir The parent directory of the source package directory. This can
 #'   be set in a global option if all your source packages are under a common
 #'   parent directory.
+#' @param timeout Timeout in seconds for \command{R CMD check}.
 #' @seealso \code{devtools::revdep_check()} is more sophisticated, but currently
 #'   has a few major issues that affect me: (1) It always deletes the
 #'   \file{*.Rcheck} directories
@@ -95,6 +96,7 @@
 #' @export
 rev_check = function(
   pkg, which = 'all', recheck = NULL, ignore = NULL, update = TRUE,
+  timeout = getOption('xfun.rev_check.timeout', 15 * 60),
   src = file.path(src_dir, pkg), src_dir = getOption('xfun.rev_check.src_dir')
 ) {
   if (length(src) != 1 || !dir.exists(src)) stop(
@@ -186,7 +188,8 @@ rev_check = function(
     check_it = function(args = NULL, ...) {
       system2(
         file.path(R.home('bin'), 'R'),
-        c(args, 'CMD', 'check', '--no-manual', shQuote(z)), stdout = FALSE, stderr = FALSE, ...
+        c(args, 'CMD', 'check', '--no-manual', shQuote(z)),
+        stdout = FALSE, stderr = FALSE, timeout = timeout, ...
       )
     }
     check_it()
@@ -323,6 +326,12 @@ download_tarball = function(p, db = available.packages(type = 'source'), dir = '
 # option xfun.install.packages
 pkg_install = function(pkgs, ...) {
   if (length(pkgs) == 0) return()
+  # in case the CRAN repo is not set up
+  repos = getOption('repos')
+  if (length(repos) == 0 || identical(repos, c(CRAN = '@CRAN@'))) {
+    opts = options(repos = c(CRAN = 'https://cran.rstudio.com'))
+    on.exit(options(opts), add = TRUE)
+  }
   install = getOption('xfun.install.packages', install.packages)
   if (length(pkgs) > 1)
     message('Installing ', length(pkgs), ' packages: ', paste(pkgs, collapse = ' '))
@@ -346,7 +355,11 @@ clean_Rcheck = function(dir, log = read_utf8(file.path(dir, '00check.log'))) {
 #'   will be converted to HTML, so you can see the diffs more clearly.
 #' @export
 compare_Rcheck = function(status_only = FALSE, output = '00check_diffs.md') {
-  if (length(dirs <- list.files('.', '.+[.]Rcheck2$')) == 0) return()
+  if (length(dirs <- list.files('.', '.+[.]Rcheck2$')) == 0) {
+    # clean up the `recheck` file
+    if (file.exists('recheck')) writeLines(character(), 'recheck')
+    return()
+  }
   d2 = function(d) c(sub('2$', '', d), d)
   logs = function(d) file.path(d2(d), '00check.log')
   dirs = dirs[rowSums(matrix(file.exists(logs(dirs)), ncol = 2)) == 2]
@@ -463,8 +476,8 @@ install_missing_latex = function() {
   tinytex::tlmgr_install(unique(pkgs))
 }
 
-# return packages that haven't been updated for X days, and can be updated on CRAN
-cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
+# retrieve the release dates of packages
+cran_pkg_dates = function(full = FALSE, maintainer = 'Yihui Xie') {
   info = tools::CRAN_package_db()
   pkgs = info[grep(maintainer, info$Maintainer), 'Package']
   info = setNames(vector('list', length(pkgs)), pkgs)
@@ -482,6 +495,12 @@ cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
     d = c(d, as.Date(gsub(r, '\\1', grep(r, x, value = TRUE))))
     info[[p]] = sort(d, decreasing = TRUE)
   }
+  if (full) info else sort(do.call(c, lapply(info, `[`, 1)), decreasing = TRUE)
+}
+
+# return packages that haven't been updated for X days, and can be updated on CRAN
+cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
+  info = cran_pkg_dates(TRUE, maintainer)
   flag = unlist(lapply(info, function(d) {
     sum(d > Sys.Date() - 180) < 6 && d[1] < Sys.Date() - days
   }))
